%% Erkennung der Druckspaltposition aus den Referenzbildern

function [nipposition] = calculatenipposition(InputPath)

%% Definitionen
% Bereiche auf dem Druckformzylinder. Bereich 1 sind die vorderen
% Testfelder, Bereich 2 sind die mittleren Testfelder, Bereich 3 sind die
% hinteren Testfelder
group = {'1', '2', '3'};
% Leeres nipposition-Struct vordefinieren, in das Ergebnisse geschrieben
% werden
nipposition = struct;


%% Berechnung der Druckspaltposition
% Schleife um alle Referenzbilder einer Kategorie (A, B und C)
for aa = 1:3
    %% Input-Ordner definieren und Liste mit Referenzbilder erstellen
    % Liste mit allen .tif-Bildern im Input-Ordner
    list = dir([InputPath filesep 'Referenzaufnahmen_Druckspalt' ...
        filesep group{aa} '*.tif']);


    %% Anzeige von Warnungen bei keinen/wenigen Referenzbildern
    % Berechnung abbrechen, wenn keine Referenzbilder vorhanden sind, 
    % Warnung ausgeben, wenn weniger als 3 Referenzbilder vorhanden sind 
    % oder Anzeigen, mit wie vielen Bilder die Referenzberechnung erfolgt.
    if isempty(list)
        % Fehler ausgeben und Berechnung abbrechen.
        error(['Es wurden keine Bilder fuer die Berechnung der '...
            'Druckspaltposition fuer Bereich ' group{aa} ' gefunden. '...
            'Verarbeitung wird abgebrochen!'])
    elseif length(list)<3
        % Warnung ausgeben
        warning(['Es wurden nur ' num2str(length(list)) ...
            ' Referenzbilder fuer Bereich ' group{aa} ' gefunden. Es ' ...
            ' ist in Betracht zu ziehen mehr Referenzbilder zu'...
            ' benutzen, um die Berechnungsgenauigkeit zu erhhen!'])
    else
        % Nachricht ausgeben, mit wie vielen Bilder die 
        % Positionsberechnung erfolgt
        msg = ['Berechnung der Druckspaltpostion fuer Bereich ' ...
            group{aa} ' wird mit ' num2str(length(list)) ...
            ' Referenzbildern durchgefuehrt.'];
        disp(msg)
        imdata = struct;
    end % if length(list)<3

    % Schleife um alle Referenzbilder eines Bereiches
    for bb = 1:length(list)
        %% Einlesen eines Referenzbilds
        % Referenzbild einlesen
        imdata(bb).refimg = ...
            imread([list(bb).folder filesep list(bb).name]);

        % Referenzbild-Informationen einlesen
        imdata(bb).refimginfo = ...
            imfinfo([list(bb).folder filesep list(bb).name]);


        %% Binarisierung vom Referenzbild (RB)
        % Referenzbild in double-Zahlenformat konvertieren und auf einen 
        % Wertebereich von 0 bis 1 mit der Bittiefe des Referenzbilds 
        % normieren
        imdata(bb).refimg = double(imdata(bb).refimg) / ...
            (2^imdata(bb).refimginfo.BitDepth - 1);

        % Referenzbild invertieren
        imdata(bb).refimg_inv = 1 - imdata(bb).refimg;

        % Referenzbild binarisieren
        imdata(bb).refimg_inv_bw = imbinarize(imdata(bb).refimg_inv);

        % Definition der maximalen Pixelzahl einer Flche, die geschlossen
        % weden soll
        MaximumPixelCount = 2000;

        % Schliessen von schwarzen Flaechen, mit einer Anzahl von unter 
        % 2000 Pixeln
        imdata(bb).refimg_inv_bw_cls = 1 - ...
            (bwareaopen(1 - imdata(bb).refimg_inv_bw, MaximumPixelCount));


        %% Ermittlung der Druckspaltposition mit Fit-Ansatz
        % leeres Bild mit Nullen fuer Kantenbild vordefineren
        imdata(bb).edge = zeros(size(imdata(bb).refimg_inv));
        % Schleife um alle Bildspalten eines Referenzbilds
        for cc = 1:size(imdata(bb).refimg_inv_bw_cls, 2)
            % Position des ersten weissen Pixels in jeder Bildspalte 
            % ermitteln
            imdata(bb).firstwhite(cc) = ...
                size(imdata(bb).refimg_inv_bw_cls,1) - ...
                find(imdata(bb).refimg_inv_bw_cls(:,cc),1);
            % Kantenbild erzeugen
            imdata(bb).edge(size(imdata(bb).refimg_inv_bw_cls,1) - ...
                imdata(bb).firstwhite(cc),cc) = 1;
        end % cc = 1:size(imdata(bb).refimg_inv_bw_cls, 2)

        % Fit an die Positionen des ersten weissen Pixels in einem Bild 
        % legen
        imdata(bb).P = polyfit(1:size(imdata(bb).refimg_inv_bw_cls, 2), ...
            [imdata(bb).firstwhite], 2);

        % Werte des Fits ermitteln
        imdata(bb).Pval = polyval(imdata(bb).P, ...
            1:size(imdata(bb).refimg_inv_bw_cls, 2));

        % Bildzeile und Bildspalte des Maximums des Fits ermitteln
        [imdata(bb).m_DS, imdata(bb).n_DS] = max([imdata(bb).Pval]);
        % Bildzeile runden damit Ergebnis ganzzahlig ist
        imdata(bb).m_DS = size(imdata(bb).refimg_inv_bw_cls, 1) - ...
            round(imdata(bb).m_DS);

        % Ermittelte Druckspaltposition in Referenzbild markieren
        imdata(bb).refimg_marked = insertMarker(imdata(bb).refimg, ...
            [imdata(bb).n_DS imdata(bb).m_DS ], 'x', 'color', 'r', ...
            'size',10);
%         % Ermittelten Fit auf Kantenbild anzeigne
%         imshow(imdata(bb).edge)
%         hold on
%         plot(1:size(imdata(bb).edge, 2), size(imdata(bb).edge, 1) ...
%             - [imdata(bb).Pval],'LineWidth',1,'Color','r')

    end % for bb = 1:length(list)
    
    %% Druckspaltposition fuer Ausgabe vorbereiten 
    % Ermittelte Druckspaltpositionen in nipposition-Struct schreiben
    nipposition.(['group_' group{aa}]).n_DS = [imdata.n_DS];
    % Mittelwert der ermittelten Druckspaltpositionen berechnen
    nipposition.(['group_' group{aa}]).n_DS_mean = mean([imdata.n_DS]);
    % Gerundeten Mittelwert der ermittelten Druckspaltpositionen berechnen
    nipposition.(['group_' group{aa}]).n_DS_mean_rnd = ...
        round(mean([imdata.n_DS]));
    % Standardabweichung der ermittelten Druckspaltpositionen berechnen
    nipposition.(['group_' group{aa}]).n_DS_std = std([imdata.n_DS]);
end % for qq = 1:3

end % function